/*
 * Copyright (c) 1998-2008 TeamDev Ltd. All Rights Reserved.
 * Use is subject to license terms.
 */
package com.teamdev.xpcom.mozilla.demo;

import java.awt.Component;
import java.awt.HeadlessException;
import java.awt.Toolkit;
import java.awt.datatransfer.DataFlavor;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;

import com.teamdev.jxbrowser.WebBrowser;
import com.teamdev.jxbrowser.event.ContextMenuEvent;
import com.teamdev.jxbrowser.event.ContextMenuListener;
import com.teamdev.xpcom.Xpcom;

/**
 * A context menu implementation object for the embedded browser. The
 * implementation provides menu items for most common actions.
 */
// serialization don't needed in demo application
@SuppressWarnings("serial")
public class ContextMenuProvider extends JPopupMenu
{
	/**
	 * Current context is not a document.
	 */
	public static final int CONTEXT_NONE = (int) ContextMenuListener.CONTEXT_NONE;

	/**
	 * Current context is a link element.
	 */
	public static final int CONTEXT_LINK = (int) ContextMenuListener.CONTEXT_LINK;

	/**
	 * Current context is an image element.
	 */
	public static final int CONTEXT_IMAGE = (int) ContextMenuListener.CONTEXT_IMAGE;

	/**
	 * Current context is a document.
	 */
	public static final int CONTEXT_DOCUMENT = (int) ContextMenuListener.CONTEXT_DOCUMENT;

	/**
	 * Current context is a text area.
	 */
	public static final int CONTEXT_TEXT = (int) ContextMenuListener.CONTEXT_TEXT;

	/**
	 * Current context is an input element.
	 */
	public static final int CONTEXT_INPUT = (int) ContextMenuListener.CONTEXT_INPUT;

	/**
	 * Current context is a background image.
	 */
	public static final int CONTEXT_BACKGROUND_IMAGE = (int) ContextMenuListener.CONTEXT_BACKGROUND_IMAGE;

    public static final int BACKWARD_ACTION = 0;
    public static final int FORWARD_ACTION = 1;
    public static final int REFRESH_ACTION = 2;
    public static final int STOP_ACTION = 3;

    private Action[] actions;
	private final JMenuItem _back = new JMenuItem("Back");

	private final JMenuItem _forward = new JMenuItem("Forward");

	private final JMenuItem _refresh = new JMenuItem("Refresh");

	private final JMenuItem _stop = new JMenuItem("Stop");

	private final JMenuItem _openLinkInNewTab = new JMenuItem("Open Link in New Tab");

	private final JMenuItem _imageProperties = new JMenuItem("Properties");

	private final JMenuItem _linkProperties = new JMenuItem("Properties");

	private final JMenuItem _linkImageProperties = new JMenuItem("Properties");

	private final JMenuItem _saveImageAs = new JMenuItem("Save Image As...");

	private final JMenuItem _noContextActions = new JMenuItem("Context Actions Unavailable");

	private AbstractAction _sourceAction;

	final private WebBrowser _browser;

	private ContextMenuEvent _contextMenuEvent;

    public ContextMenuProvider(WebBrowser webBrowser, Action[] actions) throws HeadlessException
	{
		_browser = webBrowser;
        this.actions = actions;
		_noContextActions.setEnabled(false);
		_back.addActionListener(actions[BACKWARD_ACTION]);
		_forward.addActionListener(actions[FORWARD_ACTION]);
		_refresh.addActionListener(actions[REFRESH_ACTION]);
		_stop.addActionListener(actions[STOP_ACTION]);

		_openLinkInNewTab.addActionListener(new DefaultAbstractAction());
		_imageProperties.addActionListener(new DefaultAbstractAction());
		_linkProperties.addActionListener(new DefaultAbstractAction());
		_saveImageAs.addActionListener(new DefaultAbstractAction());
		_linkImageProperties.addActionListener(new DefaultAbstractAction());
	}

	@Override
    public void setVisible(boolean stateVisible)
	{
		super.setVisible(stateVisible);
		if (!stateVisible)
		{
			_browser.activate();
		}
	}

	@Override
    public void show(final Component invoker, final int x, final int y)
	{
		int contextFlags = (int) _contextMenuEvent.getContextFlags();
		if (contextFlags == CONTEXT_TEXT || contextFlags == CONTEXT_TEXT + CONTEXT_INPUT)
		{
			_browser.deactivate();
			Xpcom.invokeAndWait(new Runnable()
			{
				public void run()
				{
					showPopupMenu(invoker, x, y);
				}
			});
			_browser.getClipboardSupport().setDisplaySelection(_contextMenuEvent.getTargetNode());
		}
		else
		{
			showPopupMenu(invoker, x, y);
		}
	}

	private void showPopupMenu(Component invoker, int x, int y)
	{
		super.show(invoker, x, y);
	}

	public ContextMenuEvent getContextMenuEvent()
	{
		return _contextMenuEvent;
	}

	public void setContextMenuEvent(ContextMenuEvent contextMenuEvent)
	{
		_contextMenuEvent = contextMenuEvent;
	}

	/**
	 * Adds an action to be executed when <i>Open Link in New Tab</i> command is
	 * selected.
	 */
	public void addOpenLinkInNewTabAction(ActionListener action)
	{
		_openLinkInNewTab.addActionListener(action);
	}

	/**
	 * Adds an action to be executed when <i>Save Image As</i> command is
	 * selected.
	 */
	public void addSaveImageAs(ActionListener action)
	{
		_saveImageAs.addActionListener(action);
	}

	/**
	 * Adds an action to be executed when <i>Properties</i> command is selected
	 * on a link element.
	 */
	public void addLinkPropertiesAction(ActionListener action)
	{
		_linkProperties.addActionListener(action);
	}

	/**
	 * Adds an action to be executed when <i>Properties</i> command is selected
	 * on an image element.
	 */
	public void addImagePropertiesAction(ActionListener action)
	{
		_imageProperties.addActionListener(action);
	}

	/**
	 * Adds an action to be executed when <i>Properties</i> command is selected
	 * on an image link element.
	 */
	public void addLinkImagePropertiesAction(ActionListener action)
	{
		_linkImageProperties.addActionListener(action);
	}

	public void addSourceAction(ActionListener action)
	{
		_sourceAction = (AbstractAction) action;
	}

	/**
	 * Configures this menu in a given context.
	 */
	public void createMenu()
	{
		removeAll();

		final int contextFlags = (int) _contextMenuEvent.getContextFlags();
		switch (contextFlags)
		{
		case CONTEXT_TEXT:
		case CONTEXT_TEXT + CONTEXT_INPUT:
			add(new JMenuItem(new CutAction()));
			add(new JMenuItem(new CopyAction(contextFlags)));
			add(new JMenuItem(new PasteAction()));
			add(new JMenuItem(new DeleteAction()));
			addSeparator();
			add(new JMenuItem(new SelectAllAction(contextFlags)));
			break;
		case CONTEXT_LINK:
			add(_openLinkInNewTab);
			addSeparator();
			add(_linkProperties);
			break;
		case CONTEXT_IMAGE:
			add(_saveImageAs);
			addSeparator();
			add(_imageProperties);
			break;
		case CONTEXT_LINK + CONTEXT_IMAGE:
			add(_openLinkInNewTab);
			addSeparator();
			add(_saveImageAs);
			addSeparator();
			add(_linkImageProperties);
			break;
		case CONTEXT_NONE:
			break;
		default:
			add(_back).setEnabled(actions[BACKWARD_ACTION].isEnabled());
			add(_forward).setEnabled(actions[FORWARD_ACTION].isEnabled());
			add(_refresh).setEnabled(actions[REFRESH_ACTION].isEnabled());
			add(_stop).setEnabled(actions[STOP_ACTION].isEnabled());
			addSeparator();
			add(new JMenuItem(new CopyAction(contextFlags)));
			add(new JMenuItem(new SelectAllAction(contextFlags)));
			addSeparator();
			add(new JMenuItem(_sourceAction));
			break;
		}
	}
	
	private class SelectAllAction extends AbstractAction
	{
		private final int _contexFlag;

		public SelectAllAction(int contextFlags)
		{
			super("Select All");
			_contexFlag = contextFlags;
		}

		public void actionPerformed(ActionEvent event)
		{
			if (_contexFlag == CONTEXT_TEXT || _contexFlag == CONTEXT_TEXT + CONTEXT_INPUT)
			{
				_browser.getClipboardSupport().selectAll(_contextMenuEvent.getTargetNode());
			}
			else
			{
				_browser.getClipboardSupport().selectAll();
			}
		}
	}

    private class CutAction extends AbstractAction
	{
		public CutAction()
		{
			super("Cut");
			setEnabled(_browser.getClipboardSupport().canSelection());
		}

		public void actionPerformed(ActionEvent event)
		{
			_browser.getClipboardSupport().cutSelection(_contextMenuEvent.getTargetNode());
		}
	}
	
	private class CopyAction extends AbstractAction
	{
		private final int _contexFlag;

		public CopyAction(int contextFlags)
		{
			super("Copy");
			_contexFlag = contextFlags;
			setEnabled(_browser.getClipboardSupport().canSelection());
		}

		public void actionPerformed(ActionEvent event)
		{
			if (_contexFlag == CONTEXT_TEXT || _contexFlag == CONTEXT_TEXT + CONTEXT_INPUT)
			{
				_browser.getClipboardSupport().copySelectionToClipboard(_contextMenuEvent.getTargetNode());
			}
			else
			{
				_browser.getClipboardSupport().copySelectionToClipboard();
			}

		}
	}
	
	private class PasteAction extends AbstractAction
	{
		public PasteAction()
		{
			super("Paste");
			setEnabled(Toolkit.getDefaultToolkit().getSystemClipboard().isDataFlavorAvailable(DataFlavor.stringFlavor));
		}

		public void actionPerformed(ActionEvent event)
		{
			_browser.getClipboardSupport().pasteClipboardContent(_contextMenuEvent.getTargetNode());
		}
	}
	
	private class DeleteAction extends AbstractAction
	{
		public DeleteAction()
		{
			super("Delete");
			setEnabled(_browser.getClipboardSupport().canSelection());
		}

		public void actionPerformed(ActionEvent event)
		{
			_browser.getClipboardSupport().deleteSelection(_contextMenuEvent.getTargetNode());
		}
	}
	
	class DefaultAbstractAction extends AbstractAction
	{
	    // pure virtual function
		public void actionPerformed(ActionEvent event)
		{
		}

		@Override
        public boolean isEnabled()
		{
			return false;
		}
	}
}
